@@ -8,6 +8,8 @@ module Net; module SSH
   #
   # Only a subset of OpenSSH configuration options are understood:
   #
+  # * ChallengeResponseAuthentication => maps to the :auth_methods option challenge-response (then coleasced into keyboard-interactive)
+  # * KbdInteractiveAuthentication => maps to the :auth_methods keyboard-interactive
   # * Ciphers => maps to the :encryption option
   # * Compression => :compression
   # * CompressionLevel => :compression_level
@@ -21,10 +23,11 @@ module Net; module SSH
   # * IdentityFile => maps to the :keys option
   # * IdentitiesOnly => :keys_only
   # * Macs => maps to the :hmac option
-  # * PasswordAuthentication => maps to the :auth_methods option
+  # * PasswordAuthentication => maps to the :auth_methods option password
   # * Port => :port
   # * PreferredAuthentications => maps to the :auth_methods option
   # * ProxyCommand => maps to the :proxy option
+  # * PubKeyAuthentication => maps to the :auth_methods option
   # * RekeyLimit => :rekey_limit
   # * User => :user
   # * UserKnownHostsFile => :user_known_hosts_file
@@ -35,6 +38,11 @@ module Net; module SSH
   class Config
     class << self
       @@default_files = %w(~/.ssh/config /etc/ssh_config /etc/ssh/ssh_config)
+      # The following defaults follow the openssh client ssh_config defaults.
+      # http://lwn.net/Articles/544640/
+      # "hostbased" is off and "none" is not supported but we allow it since
+      # it's used by some clients to query the server for allowed auth methods
+      @@default_auth_methods = %w(none publickey password keyboard-interactive)
 
       # Returns an array of locations of OpenSSH configuration files
       # to parse by default.
@@ -42,12 +50,18 @@ module Net; module SSH
         @@default_files
       end
 
+      def default_auth_methods
+        @@default_auth_methods
+      end
+
       # Loads the configuration data for the given +host+ from all of the
       # given +files+ (defaulting to the list of files returned by
       # #default_files), translates the resulting hash into the options
       # recognized by Net::SSH, and returns them.
       def for(host, files=default_files)
-        translate(files.inject({}) { |settings, file| load(file, host, settings) })
+        hash = translate(files.inject({}) { |settings, file|
+          load(file, host, settings)
+        })
       end
 
       # Load the OpenSSH configuration settings in the given +file+ for the
@@ -59,14 +73,14 @@ module Net; module SSH
       def load(path, host, settings={})
         file = File.expand_path(path)
         return settings unless File.readable?(file)
-        
+
         globals = {}
         matched_host = nil
         multi_host = []
         seen_host = false
         IO.foreach(file) do |line|
           next if line =~ /^\s*(?:#.*)?$/
-          
+
           if line =~ /^\s*(\S+)\s*=(.*)$/
             key, value = $1, $2
           else
@@ -78,19 +92,29 @@ module Net; module SSH
 
           key.downcase!
           value = $1 if value =~ /^"(.*)"$/
-          
+
           value = case value.strip
             when /^\d+$/ then value.to_i
             when /^no$/i then false
             when /^yes$/i then true
             else value
             end
-          
+
           if key == 'host'
             # Support "Host host1 host2 hostN".
             # See http://github.com/net-ssh/net-ssh/issues#issue/6
-            multi_host = value.to_s.split(/\s+/)
-            matched_host = multi_host.select { |h| host =~ pattern2regex(h) }.first
+            negative_hosts, positive_hosts = value.to_s.split(/\s+/).partition { |h| h.start_with?('!') }
+
+            # Check for negative patterns first. If the host matches, that overrules any other positive match.
+            # The host substring code is used to strip out the starting "!" so the regexp will be correct.
+            negative_match = negative_hosts.select { |h| host =~ pattern2regex(h[1..-1]) }.first
+
+            if negative_match
+              matched_host = nil
+            else
+              matched_host = positive_hosts.select { |h| host =~ pattern2regex(h) }.first
+            end
+
             seen_host = true
             settings[key] = host
           elsif !seen_host
@@ -107,9 +131,9 @@ module Net; module SSH
             end
           end
         end
-        
+
         settings = globals.merge(settings) if globals
-        
+
         return settings
       end
 
@@ -118,7 +142,9 @@ module Net; module SSH
       # +settings+ hash must have Strings for keys, all downcased, and
       # the returned hash will have Symbols for keys.
       def translate(settings)
-        settings.inject({}) do |hash, (key, value)|
+        auth_methods = default_auth_methods.clone
+        (auth_methods << 'challenge-response').uniq!
+        ret = settings.inject({:auth_methods=>auth_methods}) do |hash, (key, value)|
           case key
           when 'bindaddress' then
             hash[:bind_address] = value
@@ -138,8 +164,9 @@ module Net; module SSH
             hash[:global_known_hosts_file] = value
           when 'hostbasedauthentication' then
             if value
-              hash[:auth_methods] ||= []
-              hash[:auth_methods] << "hostbased"
+              (hash[:auth_methods] << "hostbased").uniq!
+            else
+              hash[:auth_methods].delete("hostbased")
             end
           when 'hostkeyalgorithms' then
             hash[:host_key] = value.split(/,/)
@@ -153,22 +180,36 @@ module Net; module SSH
             hash[:hmac] = value.split(/,/)
           when 'passwordauthentication'
             if value
-              hash[:auth_methods] ||= []
-              hash[:auth_methods] << "password"
+              (hash[:auth_methods] << 'password').uniq!
+            else
+              hash[:auth_methods].delete('password')
+            end
+          when 'challengeresponseauthentication'
+            if value
+              (hash[:auth_methods] << 'challenge-response').uniq!
+            else
+              hash[:auth_methods].delete('challenge-response')
+            end
+          when 'kbdinteractiveauthentication'
+            if value
+              (hash[:auth_methods] << 'keyboard-interactive').uniq!
+            else
+              hash[:auth_methods].delete('keyboard-interactive')
             end
           when 'port'
             hash[:port] = value
           when 'preferredauthentications'
-            hash[:auth_methods] = value.split(/,/)
+            hash[:auth_methods] = value.split(/,/) # TODO we should place to preferred_auth_methods rather than auth_methods
           when 'proxycommand'
             if value and !(value =~ /^none$/)
               require 'net/ssh/proxy/command'
               hash[:proxy] = Net::SSH::Proxy::Command.new(value)
             end
-	  when 'pubkeyauthentication'
+	        when 'pubkeyauthentication'
             if value
-              hash[:auth_methods] ||= []
-              hash[:auth_methods] << "publickey"
+              (hash[:auth_methods] << 'publickey').uniq!
+            else
+              hash[:auth_methods].delete('publickey')
             end
           when 'rekeylimit'
             hash[:rekey_limit] = interpret_size(value)
@@ -176,9 +217,13 @@ module Net; module SSH
             hash[:user] = value
           when 'userknownhostsfile'
             hash[:user_known_hosts_file] = value
+          when 'sendenv'
+            multi_send_env = value.to_s.split(/\s+/)
+            hash[:send_env] = multi_send_env.map { |e| Regexp.new pattern2regex(e).source, false }
           end
           hash
         end
+        merge_challenge_response_with_keyboard_interactive(ret)
       end
 
       private
@@ -202,6 +247,14 @@ module Net; module SSH
           else size.to_i
           end
         end
+        
+        def merge_challenge_response_with_keyboard_interactive(hash)
+          if hash[:auth_methods].include?('challenge-response')
+            hash[:auth_methods].delete('challenge-response')
+            (hash[:auth_methods] << 'keyboard-interactive').uniq!
+          end
+          hash
+        end
     end
   end
 
