@@ -47,8 +47,12 @@ module Net; module SSH; module Service
     # If three arguments are given, it is as if the local bind address is
     # "127.0.0.1", and the rest are applied as above.
     #
+    # To request an ephemeral port on the remote server, provide 0 (zero) for
+    # the port number. In all cases, this method will return the port that
+    # has been assigned.
+    #
     #   ssh.forward.local(1234, "www.capify.org", 80)
-    #   ssh.forward.local("0.0.0.0", 1234, "www.capify.org", 80)
+    #   assigned_port = ssh.forward.local("0.0.0.0", 0, "www.capify.org", 80)
     def local(*args)
       if args.length < 3 || args.length > 4
         raise ArgumentError, "expected 3 or 4 parameters, got #{args.length}"
@@ -69,6 +73,7 @@ module Net; module SSH; module Service
         end
       end
 
+      local_port = socket.addr[1] if local_port == 0 # ephemeral port was requested
       remote_host = args.shift
       remote_port = args.shift.to_i
 
@@ -89,6 +94,8 @@ module Net; module SSH; module Service
           channel[:socket].close
         end
       end
+      
+      local_port
     end
 
     # Terminates an active local forwarded port. If no such forwarded port
@@ -120,15 +127,21 @@ module Net; module SSH; module Service
     # forwarded immediately. If the remote server is not able to begin the
     # listener for this request, an exception will be raised asynchronously.
     #
-    # If you want to know when the connection is active, it will show up in the
-    # #active_remotes list. If you want to block until the port is active, you
-    # could do something like this:
+    # To request an ephemeral port on the remote server, provide 0 (zero) for
+    # the port number. The assigned port will show up in the # #active_remotes
+    # list.
     #
-    #   ssh.forward.remote(80, "www.google.com", 1234, "0.0.0.0")
-    #   ssh.loop { !ssh.forward.active_remotes.include?([1234, "0.0.0.0"]) }
+    # If you want to block until the port is active, you could do something
+    # like this:
+    #
+    #   old_active_remotes = ssh.forward.active_remotes
+    #   ssh.forward.remote(80, "www.google.com", 0, "0.0.0.0")
+    #   ssh.loop { !(ssh.forward.active_remotes.length > old_active_remotes.length) }
+    #   assigned_port = (ssh.forward.active_remotes - old_active_remotes).first[0]
     def remote(port, host, remote_port, remote_host="127.0.0.1")
       session.send_global_request("tcpip-forward", :string, remote_host, :long, remote_port) do |success, response|
         if success
+          remote_port = response.read_long if remote_port == 0
           debug { "remote forward from remote #{remote_host}:#{remote_port} to #{host}:#{port} established" }
           @remote_forwarded_ports[[remote_port, remote_host]] = Remote.new(host, port)
         else
@@ -257,6 +270,24 @@ module Net; module SSH; module Service
         end
       end
 
+      # not a real socket, so use a simpler behaviour
+      def prepare_simple_client(client, channel, type)
+        channel[:socket] = client
+
+        channel.on_data do |ch, data|
+          ch.debug { "data:#{data.length} on #{type} forwarded channel" }
+          ch[:socket].send(data)
+        end
+
+        channel.on_process do |ch|
+          data = ch[:socket].read(8192)
+          if data
+            ch.debug { "read #{data.length} bytes from client, sending over #{type} forwarded connection" }
+            ch.send_data(data)
+          end
+        end
+      end
+
       # The callback used when a new "forwarded-tcpip" channel is requested
       # by the server.  This will open a new socket to the host/port specified
       # when the forwarded connection was first requested.
@@ -287,7 +318,11 @@ module Net; module SSH; module Service
 
         begin
           agent = Authentication::Agent.connect(logger)
-          prepare_client(agent.socket, channel, :agent)
+          if (agent.socket.is_a? ::IO)
+            prepare_client(agent.socket, channel, :agent)
+          else
+            prepare_simple_client(agent.socket, channel, :agent)
+          end
         rescue Exception => e
           error { "attempted to connect to agent but failed: #{e.class.name} (#{e.message})" }
           raise Net::SSH::ChannelOpenFailed.new(2, "could not connect to authentication agent")
