@@ -35,6 +35,12 @@ class TestConfig < Test::Unit::TestCase
     assert !config.key?('rekeylimit')
   end
 
+  def test_load_with_wild_card_and_negative_pattern_does_not_match
+    config = Net::SSH::Config.load(config(:negative_match), "test.host")
+    assert_equal 9876, config['port']
+    assert !config.key?('compression')
+  end
+
   def test_for_should_load_all_files_and_translate_to_net_ssh_options
     config = Net::SSH::Config.for("test.host", [config(:exact_match), config(:wild_cards)])
     assert_equal 1234, config[:port]
@@ -86,7 +92,8 @@ class TestConfig < Test::Unit::TestCase
       'passwordauthentication'  => true,
       'port'                    => 1234,
       'pubkeyauthentication'    => true,
-      'rekeylimit'              => 1024
+      'rekeylimit'              => 1024,
+      'sendenv'                 => "LC_*"
     }
 
     net_ssh = Net::SSH::Config.translate(open_ssh)
@@ -96,13 +103,76 @@ class TestConfig < Test::Unit::TestCase
     assert_equal 6,         net_ssh[:compression_level]
     assert_equal 100,       net_ssh[:timeout]
     assert_equal true,      net_ssh[:forward_agent]
-    assert_equal %w(hostbased password publickey), net_ssh[:auth_methods].sort
+    assert_equal %w(hostbased keyboard-interactive none password publickey), net_ssh[:auth_methods].sort
     assert_equal %w(d e f), net_ssh[:host_key]
     assert_equal %w(g h i), net_ssh[:keys]
     assert_equal %w(j k l), net_ssh[:hmac]
     assert_equal 1234,      net_ssh[:port]
     assert_equal 1024,      net_ssh[:rekey_limit]
     assert_equal "127.0.0.1", net_ssh[:bind_address]
+    assert_equal [/^LC_.*$/], net_ssh[:send_env]
+  end
+
+  def test_translate_should_turn_off_authentication_methods
+    open_ssh = {
+      'hostbasedauthentication'         => false,
+      'passwordauthentication'          => false,
+      'pubkeyauthentication'            => false,
+      'challengeresponseauthentication' => false,
+      'kbdinteractiveauthentication'    => false
+    }
+
+    net_ssh = Net::SSH::Config.translate(open_ssh)
+
+    assert_equal %w(none), net_ssh[:auth_methods].sort
+  end
+
+  def test_translate_should_turn_on_authentication_methods
+    open_ssh = {
+      'hostbasedauthentication'         => true,
+      'passwordauthentication'          => true,
+      'pubkeyauthentication'            => true,
+      'challengeresponseauthentication' => true,
+      'kbdinteractiveauthentication'    => true
+    }
+
+    net_ssh = Net::SSH::Config.translate(open_ssh)
+
+    assert_equal %w(hostbased keyboard-interactive none password publickey), net_ssh[:auth_methods].sort
+  end
+
+  def test_translate_should_not_disable_keyboard_interactive_when_challange_or_keyboardinterective_is_on
+    open_ssh = {
+      'kbdinteractiveauthentication' => false
+    }
+    net_ssh = Net::SSH::Config.translate(open_ssh)
+    assert_equal %w(keyboard-interactive none password publickey), net_ssh[:auth_methods].sort
+
+    open_ssh = {
+      'challengeresponseauthentication' => false
+    }
+    net_ssh = Net::SSH::Config.translate(open_ssh)
+    assert_equal %w(keyboard-interactive none password publickey), net_ssh[:auth_methods].sort
+  end
+  
+  def test_should_ddisable_keyboard_interactive_when_challeng_and_keyboardinteractive_is_off
+    open_ssh = {
+      'challengeresponseauthentication' => false,
+      'kbdinteractiveauthentication' => false
+    }
+
+    net_ssh = Net::SSH::Config.translate(open_ssh)
+    assert_equal %w(none password publickey), net_ssh[:auth_methods].sort
+  end
+
+  def test_for_should_turn_off_authentication_methods
+    config = Net::SSH::Config.for("test.host", [config(:empty), config(:auth_off), config(:auth_on)])
+    assert_equal %w(none), config[:auth_methods].sort
+  end
+
+  def test_for_should_turn_on_authentication_methods
+    config = Net::SSH::Config.for("test.host", [config(:empty), config(:auth_on), config(:auth_off)])
+    assert_equal %w(hostbased keyboard-interactive none password publickey), config[:auth_methods].sort
   end
   
   def test_load_with_plus_sign_hosts
@@ -134,7 +204,13 @@ class TestConfig < Test::Unit::TestCase
     net_ssh = Net::SSH::Config.translate(config)
     assert_equal 'prefix.1234.sufix', net_ssh[:host_name]
   end
-  
+
+  def test_load_with_send_env
+    config = Net::SSH::Config.load(config(:send_env), "1234")
+    net_ssh = Net::SSH::Config.translate(config)
+    assert_equal [/^GIT_.*$/, /^LANG$/, /^LC_.*$/], net_ssh[:send_env]
+  end
+
   private
 
     def config(name)
